/* $Id: ast_functiondefinition.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_FUNCTIONDEFINITION_H
#define WTPARSER_AST_FUNCTIONDEFINITION_H

#include "../astnode.h"
#include "ast_block.h"
#include "welltype/wtruntime.h"

namespace wtast
{

class FunctionSignature;
class PreCondition;
class PostCondition;
class ExceptionCompound;

class FunctionDefinition : public Block
{
	FunctionSignature* mFuncSig;
	PreCondition*      mPreCond;
	PostCondition*     mPostCond;
	ExceptionCompound* mBody;

public:
	FunctionDefinition(FunctionSignature* sig, PreCondition* precond, PostCondition* postcond, ExceptionCompound* ec)
		: Block()
		, mFuncSig(sig)
		, mPreCond(precond)
		, mPostCond(postcond)
		, mBody(ec)
	{
		// ok
	}

	~FunctionDefinition();

	void Accept(AstVisitor* vis);

	FunctionDefinition* Copy();

	Kind GetKind() const
	{
		return FUNCTIONDEFINITION;
	}

	FunctionSignature* GetFunctionSignature()
	{
		return mFuncSig;
	}

	PreCondition* GetPreCondition()
	{
		return mPreCond;
	}

	PostCondition* GetPostCondition()
	{
		return mPostCond;
	}

	ExceptionCompound* GetBody()
	{
		return mBody;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_FUNCTIONDEFINITION_H
