/* $Id: ast_recordtype.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_RECORDTYPE_H
#define WTPARSER_AST_RECORDTYPE_H

#include "../astnode.h"
#include "welltype/wtruntime.h"
#include "ast_type.h"
#include <deque>
#include <map>
#include <string>

namespace wtast
{

class RecordField;

class RecordType : public Type
{
	typedef std::deque<RecordField*> fields_t;
	typedef std::map<std::string, RecordField*> fieldmap_t;
	fields_t    mFields;
	fieldmap_t  mFieldMap;
	std::string mName;
	wt_u32      mTypeId;
	wt_u32      mTotalSize;
	bool        mPrivate;
	bool        mLimited;
	bool        mRestricted;

public:
	RecordType(const std::string & name, wt_u32 type_id, bool priv, bool lim, bool restricted)
		: Type()
		, mName(name)
		, mTypeId(type_id)
		, mTotalSize(0)
		, mPrivate(priv)
		, mLimited(lim)
		, mRestricted(restricted)
	{
		// ok
	}

	~RecordType();

	void Accept(AstVisitor* vis);

	RecordType* Copy();

	Kind GetKind() const
	{
		return RECORDTYPE;
	}

	std::string GetName() const
	{
		return mName;
	}

	wt_u32 GetTypeId() const
	{
		return mTypeId;
	}

	wt_u32 GetLoadSize() const;

	bool IsObject() const
	{
		return true;
	}

	bool IsDefaultConstructible() const;

	bool HasFunctionType() const;

	bool IsCopyable() const;

	wt_u32 GetTotalSize() const
	{
		return mTotalSize;
	}

	bool AddField(RecordField* field);

	std::size_t GetNumberOfFields() const
	{
		return mFields.size();
	}

	RecordField* GetField(std::size_t idx)
	{
		if(idx<mFields.size())
		{
			return mFields[idx];
		}
		else
		{
			return 0;
		}
	}

	RecordField* GetField(const std::string & fieldname)
	{
		fieldmap_t::iterator it = mFieldMap.find(fieldname);

		if(mFieldMap.end()!=it)
		{
			return it->second;
		}

		return 0;
	}

	void SetMutable()
	{
		mTypeId |= WTTF_MUTABLE;
	}

	bool IsPrivate() const
	{
		return mPrivate;
	}

	bool IsLimited() const
	{
		return mLimited;
	}

	bool IsRestricted() const
	{
		return mRestricted;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_RECORDTYPE_H
