 // $Id: lexer.l 4993 2016-09-23 20:19:19Z aron $
%option noyywrap c++ yylineno prefix="yyWT"

%{
#include "parser.tab.hpp"
#include "ParserHelper.h"

void WTFlexLexer_UpdatePositionInfo(WTFlexLexer* lexer, unsigned int len);
void WTFlexLexer_NewLine(WTFlexLexer* lexer);
void WTFlexLexer_WhiteSpaces(WTFlexLexer* lexer, const char* str, unsigned int len);

#define YY_USER_ACTION do \
	{ \
		WTFlexLexer_UpdatePositionInfo(reinterpret_cast<WTFlexLexer*>(this), yyleng); \
	} \
	while(0);
%}

modifier_B      [bB]
modifier_F      [fF]
modifier_H      [hH]
modifier_I      [iI]
modifier_L      [lL]
modifier_O      [oO]
modifier_U      [uU]
modifier_X      [xX]
letter          [a-zA-Z]
whitespace      [ \t\r\n]
whitespaces     {whitespace}+
whitespace_any  {whitespace}*
ident_start     {letter}|"_"
ident_any       {ident_start}|{digitbase_dec}
ident           {ident_start}{ident_any}*
digitbase_bin   [01]
digitbase_oct   [0-7]
digitbase_dec   [0-9]
digitbase_hex   [0-9a-fA-F]
digit_bin       _|{digitbase_bin}
digit_oct       _|{digitbase_oct}
digit_dec       _|{digitbase_dec}
digit_hex       _|{digitbase_hex}
number_bin      0{modifier_B}{digit_bin}+
number_oct      0{modifier_O}{digit_oct}+
number_dec      {digit_dec}+
number_hex      0{modifier_X}{digit_hex}+
number_double   {digit_dec}+"."{digit_dec}+
number_float    {digit_dec}+"."{digit_dec}*{modifier_F}
i8bin           {number_bin}{modifier_I}{modifier_B}
u8bin           {number_bin}{modifier_U}{modifier_B}
i16bin          {number_bin}{modifier_H}
u16bin          {number_bin}{modifier_U}{modifier_H}
i32bin          {number_bin}
u32bin          {number_bin}{modifier_U}
i64bin          {number_bin}{modifier_L}
u64bin          {number_bin}{modifier_U}{modifier_L}
i8oct           {number_oct}{modifier_I}{modifier_B}
u8oct           {number_oct}{modifier_U}{modifier_B}
i16oct          {number_oct}{modifier_H}
u16oct          {number_oct}{modifier_U}{modifier_H}
i32oct          {number_oct}
u32oct          {number_oct}{modifier_U}
i64oct          {number_oct}{modifier_L}
u64oct          {number_oct}{modifier_U}{modifier_L}
i8dec           {number_dec}{modifier_I}{modifier_B}
u8dec           {number_dec}{modifier_U}{modifier_B}
i16dec          {number_dec}{modifier_H}
u16dec          {number_dec}{modifier_U}{modifier_H}
i32dec          {number_dec}
u32dec          {number_dec}{modifier_U}
i64dec          {number_dec}{modifier_L}
u64dec          {number_dec}{modifier_U}{modifier_L}
i8hex           {number_hex}{modifier_I}{modifier_B}
u8hex           {number_hex}{modifier_U}{modifier_B}
i16hex          {number_hex}{modifier_H}
u16hex          {number_hex}{modifier_U}{modifier_H}
i32hex          {number_hex}
u32hex          {number_hex}{modifier_U}
i64hex          {number_hex}{modifier_L}
u64hex          {number_hex}{modifier_U}{modifier_L}
raisenear       "raise"{whitespace_any}"near"
asterix         \x2a
slash           \x2f
newline         \n
strchar         \\[trnbv0\\\'\"]|[^\\"]|\\[xX]{digitbase_hex}{2}
char            '{strchar}'
string          \"{strchar}*\"
opchar_middle   [\|\-\^\=\!\<\>\+\&\%\~]
opchar          {opchar_middle}|{asterix}*|{slash}
operator        {opchar}({opchar_middle}{opchar}?)*

%%

"^\xef\xbb\xbf"         { return CTL_UTF8_BOM; }
";"                     { return wtparser::WTParser::token::SEMICOLON; }
","                     { return wtparser::WTParser::token::COMMA; }
":"                     { return wtparser::WTParser::token::COLON; }
"::"                    { return wtparser::WTParser::token::COLONCOLON; }
"."                     { return wtparser::WTParser::token::DOT; }
".."                    { return wtparser::WTParser::token::DOTDOT; }
"..."                   { return wtparser::WTParser::token::DOTDOTDOT; }
"to"                    { return wtparser::WTParser::token::DOTDOT; }
"="|":="                { return wtparser::WTParser::token::ASSIGN; }
"("                     { return wtparser::WTParser::token::LPAR; }
")"                     { return wtparser::WTParser::token::RPAR; }
"["                     { return wtparser::WTParser::token::LBRACKET; }
"]"                     { return wtparser::WTParser::token::RBRACKET; }
"{"                     { return wtparser::WTParser::token::LBRACE; }
"}"                     { return wtparser::WTParser::token::RBRACE; }
"<"                     { return wtparser::WTParser::token::LANGLEBRACKET; }
">"                     { return wtparser::WTParser::token::RANGLEBRACKET; }
"&&&"                   { return wtparser::WTParser::token::OP_LANDTHEN; }
"|||"                   { return wtparser::WTParser::token::OP_LORELSE; }
"char"                  { return wtparser::WTParser::token::TYPE_CHAR; }
"bool"                  { return wtparser::WTParser::token::TYPE_BOOL; }
"handle"                { return wtparser::WTParser::token::TYPE_HANDLE; }
"i8"                    { return wtparser::WTParser::token::TYPE_I8; }
"byte"                  { return wtparser::WTParser::token::TYPE_I8; }
"u8"                    { return wtparser::WTParser::token::TYPE_U8; }
"ubyte"                 { return wtparser::WTParser::token::TYPE_U8; }
"i16"                   { return wtparser::WTParser::token::TYPE_I16; }
"short"                 { return wtparser::WTParser::token::TYPE_I16; }
"u16"                   { return wtparser::WTParser::token::TYPE_U16; }
"ushort"                { return wtparser::WTParser::token::TYPE_U16; }
"i32"                   { return wtparser::WTParser::token::TYPE_I32; }
"int"                   { return wtparser::WTParser::token::TYPE_I32; }
"u32"                   { return wtparser::WTParser::token::TYPE_U32; }
"uint"                  { return wtparser::WTParser::token::TYPE_U32; }
"i64"                   { return wtparser::WTParser::token::TYPE_I64; }
"long"                  { return wtparser::WTParser::token::TYPE_I64; }
"u64"                   { return wtparser::WTParser::token::TYPE_U64; }
"ulong"                 { return wtparser::WTParser::token::TYPE_U64; }
"f32"                   { return wtparser::WTParser::token::TYPE_F32; }
"float"                 { return wtparser::WTParser::token::TYPE_F32; }
"f64"                   { return wtparser::WTParser::token::TYPE_F64; }
"double"                { return wtparser::WTParser::token::TYPE_F64; }
"seq"                   { return wtparser::WTParser::token::KW_SEQ; }
"string"                { return wtparser::WTParser::token::TYPE_STRING; }
"file"                  { return wtparser::WTParser::token::TYPE_FILE; }
"dir"                   { return wtparser::WTParser::token::TYPE_DIR; }
"socket"                { return wtparser::WTParser::token::TYPE_SOCKET; }
"binary"                { return CTL_RESERVED_TOKEN; /*return wtparser::WTParser::token::TYPE_BINARY;*/ }
"wchar"                 { return CTL_RESERVED_TOKEN; /*return wtparser::WTParser::token::TYPE_WCHAR;*/ }
"wstring"               { return CTL_RESERVED_TOKEN; /*return wtparser::WTParser::token::TYPE_WSTRING;*/ }
"#datastacksize"        { return wtparser::WTParser::token::OPT_DSS; }
"#callstacksize"        { return wtparser::WTParser::token::OPT_CSS; }
"#once"                 { return CTL_ONCE; }
"#include"              { return CTL_INCLUDE; }
"#meta:author"          { return wtparser::WTParser::token::META_AUTHOR; }
"#meta:title"           { return wtparser::WTParser::token::META_TITLE; }
"#meta:version"         { return wtparser::WTParser::token::META_VERSION; }
"#meta:copyright"       { return wtparser::WTParser::token::META_COPYRIGHT; }
"#meta:id"              { return wtparser::WTParser::token::META_ID; }
"#meta:comment"         { return wtparser::WTParser::token::META_COMMENT; }
"#meta:*"               { return wtparser::WTParser::token::META__CUSTOM; }
"true"                  { return wtparser::WTParser::token::KW_TRUE; }
"false"                 { return wtparser::WTParser::token::KW_FALSE; }
"do"                    { return wtparser::WTParser::token::KW_DO; }
"if"                    { return wtparser::WTParser::token::KW_IF; }
"else"                  { return wtparser::WTParser::token::KW_ELSE; }
"elif"                  { return wtparser::WTParser::token::KW_ELIF; }
"while"                 { return wtparser::WTParser::token::KW_WHILE; }
"for"                   { return wtparser::WTParser::token::KW_FOR; }
"foreach"               { return wtparser::WTParser::token::KW_FOREACH; }
"return"                { return wtparser::WTParser::token::KW_RETURN; }
"function"              { return wtparser::WTParser::token::KW_FUNCTION; }
"exception"             { return wtparser::WTParser::token::KW_EXCEPTION; }
"raise"                 { return wtparser::WTParser::token::KW_RAISE; }
{raisenear}             { return wtparser::WTParser::token::KW_RAISENEAR; }
"finally"               { return wtparser::WTParser::token::KW_FINALLY; }
"main"                  { return wtparser::WTParser::token::KW_MAIN; }
"ctor"                  { return wtparser::WTParser::token::KW_CTOR; }
"dtor"                  { return wtparser::WTParser::token::KW_DTOR; }
"global"                { return wtparser::WTParser::token::KW_GLOBAL; }
"import"                { return wtparser::WTParser::token::KW_IMPORT; }
"export"                { return wtparser::WTParser::token::KW_EXPORT; }
"declare"               { return wtparser::WTParser::token::KW_DECLARE; }
"mutable"               { return wtparser::WTParser::token::KW_MUTABLE; }
"inclusive"             { return wtparser::WTParser::token::KW_INCLUSIVE; }
"exclusive"             { return wtparser::WTParser::token::KW_EXCLUSIVE; }
"reverse"               { return wtparser::WTParser::token::KW_REVERSE; }
"which"                 { return wtparser::WTParser::token::KW_WHICH; }
"operator"              { return wtparser::WTParser::token::KW_OPERATOR; }
"in"                    { return wtparser::WTParser::token::KW_IN; }
"record"                { return wtparser::WTParser::token::KW_RECORD; }
"pre"                   { return wtparser::WTParser::token::KW_PRE; }
"post"                  { return wtparser::WTParser::token::KW_POST; }
"assert"                { return wtparser::WTParser::token::KW_ASSERT; }
"limited"               { return wtparser::WTParser::token::KW_LIMITED; }
"private"               { return wtparser::WTParser::token::KW_PRIVATE; }
"protected"             { return CTL_RESERVED_TOKEN; /*return wtparser::WTParser::token::KW_PROTECTED;*/ }
"public"                { return CTL_RESERVED_TOKEN; /*return wtparser::WTParser::token::KW_PUBLIC;*/ }
"alias"                 { return wtparser::WTParser::token::KW_ALIAS; }
"pure"                  { return wtparser::WTParser::token::KW_PURE; }
"impure"                { return wtparser::WTParser::token::KW_IMPURE; }
"new"                   { return CTL_RESERVED_TOKEN; }
"old"                   { return wtparser::WTParser::token::KW_OLD; }
"datatype"              { return wtparser::WTParser::token::KW_DATATYPE; }
"enum"                  { return wtparser::WTParser::token::KW_ENUM; }
"switch"                { return wtparser::WTParser::token::KW_SWITCH; }
"case"                  { return wtparser::WTParser::token::KW_CASE; }
"default"               { return wtparser::WTParser::token::KW_DEFAULT; }
"break"                 { return wtparser::WTParser::token::KW_BREAK; }
"type"                  { return wtparser::WTParser::token::KW_TYPE; }
"out"                   { return CTL_RESERVED_TOKEN; }
"atomic"                { return CTL_RESERVED_TOKEN; }
"thread"                { return CTL_RESERVED_TOKEN; }
"task"                  { return CTL_RESERVED_TOKEN; }
"lock"                  { return CTL_RESERVED_TOKEN; }
"unlock"                { return CTL_RESERVED_TOKEN; }
"subtype"               { return CTL_RESERVED_TOKEN; }
"subtype_upcast"        { return CTL_RESERVED_TOKEN; }
"subtype_downcast"      { return CTL_RESERVED_TOKEN; }
"__FILE__"              { return wtparser::WTParser::token::PREDEF_FILE; }
"__LINE__"              { return wtparser::WTParser::token::PREDEF_LINE; }
"__DATE__"              { return wtparser::WTParser::token::PREDEF_DATE; }
"__TIME__"              { return wtparser::WTParser::token::PREDEF_TIME; }
"__DATETIME__"          { return wtparser::WTParser::token::PREDEF_DATETIME; }

{char}                  { return wtparser::WTParser::token::CHARLIT; }
{string}                { return wtparser::WTParser::token::STRINGLIT; }
{ident}                 { return wtparser::ParserHelper::Lexer_TransformIdent(this); }
{number_float}          { return wtparser::WTParser::token::FLOATNUMBER; }
{number_double}         { return wtparser::WTParser::token::DOUBLENUMBER; }
{i8bin}                 { return wtparser::WTParser::token::I8BIN;  }
{u8bin}                 { return wtparser::WTParser::token::U8BIN;  }
{i16bin}                { return wtparser::WTParser::token::I16BIN; }
{u16bin}                { return wtparser::WTParser::token::U16BIN; }
{i32bin}                { return wtparser::WTParser::token::I32BIN; }
{u32bin}                { return wtparser::WTParser::token::U32BIN; }
{i64bin}                { return wtparser::WTParser::token::I64BIN; }
{u64bin}                { return wtparser::WTParser::token::U64BIN; }
{i8oct}                 { return wtparser::WTParser::token::I8OCT;  }
{u8oct}                 { return wtparser::WTParser::token::U8OCT;  }
{i16oct}                { return wtparser::WTParser::token::I16OCT; }
{u16oct}                { return wtparser::WTParser::token::U16OCT; }
{i32oct}                { return wtparser::WTParser::token::I32OCT; }
{u32oct}                { return wtparser::WTParser::token::U32OCT; }
{i64oct}                { return wtparser::WTParser::token::I64OCT; }
{u64oct}                { return wtparser::WTParser::token::U64OCT; }
{i8dec}                 { return wtparser::WTParser::token::I8DEC;  }
{u8dec}                 { return wtparser::WTParser::token::U8DEC;  }
{i16dec}                { return wtparser::WTParser::token::I16DEC; }
{u16dec}                { return wtparser::WTParser::token::U16DEC; }
{i32dec}                { return wtparser::WTParser::token::I32DEC; }
{u32dec}                { return wtparser::WTParser::token::U32DEC; }
{i64dec}                { return wtparser::WTParser::token::I64DEC; }
{u64dec}                { return wtparser::WTParser::token::U64DEC; }
{i8hex}                 { return wtparser::WTParser::token::I8HEX;  }
{u8hex}                 { return wtparser::WTParser::token::U8HEX;  }
{i16hex}                { return wtparser::WTParser::token::I16HEX; }
{u16hex}                { return wtparser::WTParser::token::U16HEX; }
{i32hex}                { return wtparser::WTParser::token::I32HEX; }
{u32hex}                { return wtparser::WTParser::token::U32HEX; }
{i64hex}                { return wtparser::WTParser::token::I64HEX; }
{u64hex}                { return wtparser::WTParser::token::U64HEX; }

"//".*\n                { // single-line comment
                            WTFlexLexer_NewLine(reinterpret_cast<WTFlexLexer*>(this));
                            //return CTL_COMMENT;
                        }
"/*"                    { // multi-line comment
                            char c1=0, c2=0;
                            do
                            {
                                c1 = c2;
                                c2 = yyinput();
                                if('\n'==c2)
                                {
                                    WTFlexLexer_NewLine(reinterpret_cast<WTFlexLexer*>(this));
                                }
                            }
                            while(c1!='*' || c2!='/');
                            //return CTL_COMMENT;
                        }

{operator}              { return wtparser::ParserHelper::Lexer_Operator(this); }

{whitespaces}           { // white spaces
                            WTFlexLexer_WhiteSpaces(reinterpret_cast<WTFlexLexer*>(this), yytext, yyleng);
                        }

<<EOF>>                 { return CTL_EOF; }

.                       {
                            LexerError(yytext);
                            return -1;
                        }

%%

